import * as logger from './log'
import * as Mqtt from 'mqtt'
import * as fs from 'fs'
import * as config from './config'
import { IClientPublishOptions } from 'mqtt';



export type TopicCallbackFunc = (topic: string, payload: string) => void
export type StartCallbackFunc = () => void

export interface TopicHandler {
    topic: string,
    callback: TopicCallbackFunc
}



class MqttHandler {
    private mqttClient: Mqtt.Client
    private mqttOptions: Mqtt.IClientOptions = {}
    private mqttBrokerUrl: string
    private topicHandlers: TopicHandler[]
    private startCallbacks: StartCallbackFunc[]

    constructor() {
        this.mqttBrokerUrl = config.dict.brokerUrl
        this.mqttOptions.clientId = 'dispatcher'
        if (config.dict.brokerUser && config.dict.brokerPass) {
            this.mqttOptions.username = config.dict.brokerUser
            this.mqttOptions.password = config.dict.brokerPass
        }

        if (config.dict.brokerCa) {
            this.mqttOptions.ca = fs.readFileSync(config.dict.brokerCa, 'ascii')
            this.mqttOptions.rejectUnauthorized = true
        }

        this.topicHandlers = []
        this.startCallbacks = []

        logger.info("MqttHandler constructed")
    }

    register(topics: string[], cb: TopicCallbackFunc, startCb: StartCallbackFunc) : void {
        topics.forEach((topic) => {
            this.topicHandlers.push({topic: topic, callback: cb})
            logger.info(`Callback registered for ${topic}`)
        })
        this.startCallbacks.push(startCb)
    }

    exec() : void {
        logger.info(`Connecting to ${this.mqttBrokerUrl}`)
        this.mqttClient = Mqtt.connect(this.mqttBrokerUrl, this.mqttOptions)

        this.mqttClient.on('error', (err) => {
            logger.error(`Error in mqttHandler: ${err}`)
        })
        this.mqttClient.on('connect', () : void => {
            this.mqttClient.publish('dispatcher_ng/status', 'dispatcher_ng running')
            this.mqttClient.subscribe('dispatcher_ng/cmd')
            this.topicHandlers.forEach((topicHandler) => {
                this.mqttClient.subscribe(topicHandler.topic)
                logger.info(`${topicHandler.topic} subscribed`)
            })
            logger.info('MQTT connection established')
            this.startCallbacks.forEach((cb) => { 
                cb()
                logger.info("started")
            })
        })
        this.mqttClient.on('message', (topic: string, payload: Buffer, packet : Mqtt.IPublishPacket): void => {
            // if (! packet.retain) {
                let payloadStr : string = payload.toString('UTF-8')
                // logger.info(`Message received on topic ${topic}: ${payload}`)
                this.processMessage(topic, payloadStr)
            // }
        })
    }

    processMessage(topic: string, payload: string) : boolean {
        let found = false;
        this.topicHandlers.forEach((topicHandler) => {
            // logger.warn(`Test: ${subscribedTopic}, ${topic}`);
            // console.log(`Test: ${subscribedTopic}, ${topic}`);
            if (topicHandler.topic == topic) {
                // logger.warn('1');
                topicHandler.callback(topic, payload)
                found = true
            } else if (topicHandler.topic.endsWith('#') && 
                    (topicHandler.topic.substring(0, topicHandler.topic.length-1) ==
                     topic.substring(0, topicHandler.topic.length-1))) {
                // logger.warn('2');
                // console.log('2');
                topicHandler.callback(topic, payload)
                found = true
            }
        })
        return found
    }
    
    send(topic: string, payload: string, internalFirst: boolean = false) : void {
        //let sent = false
        //if (internalFirst) {
        //    logger.info(`Try internal sending: ${topic}`)
        //    sent = this.processMessage(topic, payload)
        //}
        //if (! sent) {
            logger.info(`External sending required: ${topic} ${payload}`)
            let options : IClientPublishOptions = { retain: true, qos: 0 }
            this.mqttClient.publish(topic, payload, options)
        //} else {
        //    logger.info(`Internally delivered: ${topic}`)
        //}
    }

}

export let mqttHandler = new MqttHandler()

